@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -28,9 +28,6 @@
 #include "gpm-common.h"
 #include "gpm-point-obj.h"
 #include "gpm-graph-widget.h"
-#include "egg-debug.h"
-#include "egg-color.h"
-#include "egg-precision.h"
 
 G_DEFINE_TYPE (GpmGraphWidget, gpm_graph_widget, GTK_TYPE_DRAWING_AREA);
 #define GPM_GRAPH_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_GRAPH_WIDGET, GpmGraphWidgetPrivate))
@@ -41,6 +38,7 @@ struct GpmGraphWidgetPrivate
 	gboolean		 use_grid;
 	gboolean		 use_legend;
 	gboolean		 autorange_x;
+	gboolean		 autorange_y;
 
 	GSList			*key_data; /* lines */
 
@@ -56,8 +54,8 @@ struct GpmGraphWidgetPrivate
 	gfloat			 unit_x; /* 10th width of graph */
 	gfloat			 unit_y; /* 10th width of graph */
 
-	GpmGraphWidgetType	 axis_type_x;
-	GpmGraphWidgetType	 axis_type_y;
+	GpmGraphWidgetType	 type_x;
+	GpmGraphWidgetType	 type_y;
 	gchar			*title;
 
 	cairo_t			*cr;
@@ -67,9 +65,24 @@ struct GpmGraphWidgetPrivate
 	GPtrArray		*plot_list;
 };
 
-static gboolean gpm_graph_widget_expose (GtkWidget *graph, GdkEventExpose *event);
+static gboolean gpm_graph_widget_draw (GtkWidget *widget, cairo_t *cr);
 static void	gpm_graph_widget_finalize (GObject *object);
 
+enum
+{
+	PROP_0,
+	PROP_USE_LEGEND,
+	PROP_USE_GRID,
+	PROP_TYPE_X,
+	PROP_TYPE_Y,
+	PROP_AUTORANGE_X,
+	PROP_AUTORANGE_Y,
+	PROP_START_X,
+	PROP_START_Y,
+	PROP_STOP_X,
+	PROP_STOP_Y,
+};
+
 /**
  * gpm_graph_widget_key_data_clear:
  **/
@@ -77,13 +90,13 @@ static gboolean
 gpm_graph_widget_key_data_clear (GpmGraphWidget *graph)
 {
 	GpmGraphWidgetKeyData *keyitem;
-	guint a;
+	guint i;
 
 	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
 
 	/* remove items in list and free */
-	for (a=0; a<g_slist_length (graph->priv->key_data); a++) {
-		keyitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, a);
+	for (i=0; i<g_slist_length (graph->priv->key_data); i++) {
+		keyitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, i);
 		g_free (keyitem->desc);
 		g_free (keyitem);
 	}
@@ -103,7 +116,7 @@ gpm_graph_widget_key_data_add (GpmGraphWidget *graph, guint32 color, const gchar
 
 	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
 
-	egg_debug ("add to list %s", desc);
+	g_debug ("add to list %s", desc);
 	keyitem = g_new0 (GpmGraphWidgetKeyData, 1);
 
 	keyitem->color = color;
@@ -114,43 +127,94 @@ gpm_graph_widget_key_data_add (GpmGraphWidget *graph, guint32 color, const gchar
 }
 
 /**
- * gpm_graph_widget_set_type_x:
- * @graph: This class instance
- * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
+ * up_graph_get_property:
  **/
-void
-gpm_graph_widget_set_type_x (GpmGraphWidget *graph, GpmGraphWidgetType axis)
+static void
+up_graph_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->axis_type_x = axis;
+	GpmGraphWidget *graph = GPM_GRAPH_WIDGET (object);
+	switch (prop_id) {
+	case PROP_USE_LEGEND:
+		g_value_set_boolean (value, graph->priv->use_legend);
+		break;
+	case PROP_USE_GRID:
+		g_value_set_boolean (value, graph->priv->use_grid);
+		break;
+	case PROP_TYPE_X:
+		g_value_set_uint (value, graph->priv->type_x);
+		break;
+	case PROP_TYPE_Y:
+		g_value_set_uint (value, graph->priv->type_y);
+		break;
+	case PROP_AUTORANGE_X:
+		g_value_set_boolean (value, graph->priv->autorange_x);
+		break;
+	case PROP_AUTORANGE_Y:
+		g_value_set_boolean (value, graph->priv->autorange_y);
+		break;
+	case PROP_START_X:
+		g_value_set_int (value, graph->priv->start_x);
+		break;
+	case PROP_START_Y:
+		g_value_set_int (value, graph->priv->start_y);
+		break;
+	case PROP_STOP_X:
+		g_value_set_int (value, graph->priv->stop_x);
+		break;
+	case PROP_STOP_Y:
+		g_value_set_int (value, graph->priv->stop_y);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 /**
- * gpm_graph_widget_set_type_y:
- * @graph: This class instance
- * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
+ * up_graph_set_property:
  **/
-void
-gpm_graph_widget_set_type_y (GpmGraphWidget *graph, GpmGraphWidgetType axis)
+static void
+up_graph_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->axis_type_y = axis;
-}
+	GpmGraphWidget *graph = GPM_GRAPH_WIDGET (object);
 
-/**
- * gpm_graph_widget_enable_legend:
- * @graph: This class instance
- * @enable: If we should show the legend
- **/
-void
-gpm_graph_widget_enable_legend (GpmGraphWidget *graph, gboolean enable)
-{
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->use_legend = enable;
+	switch (prop_id) {
+	case PROP_USE_LEGEND:
+		graph->priv->use_legend = g_value_get_boolean (value);
+		break;
+	case PROP_USE_GRID:
+		graph->priv->use_grid = g_value_get_boolean (value);
+		break;
+	case PROP_TYPE_X:
+		graph->priv->type_x = g_value_get_uint (value);
+		break;
+	case PROP_TYPE_Y:
+		graph->priv->type_y = g_value_get_uint (value);
+		break;
+	case PROP_AUTORANGE_X:
+		graph->priv->autorange_x = g_value_get_boolean (value);
+		break;
+	case PROP_AUTORANGE_Y:
+		graph->priv->autorange_y = g_value_get_boolean (value);
+		break;
+	case PROP_START_X:
+		graph->priv->start_x = g_value_get_int (value);
+		break;
+	case PROP_START_Y:
+		graph->priv->start_y = g_value_get_int (value);
+		break;
+	case PROP_STOP_X:
+		graph->priv->stop_x = g_value_get_int (value);
+		break;
+	case PROP_STOP_Y:
+		graph->priv->stop_y = g_value_get_int (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 
+	/* refresh widget */
 	gtk_widget_hide (GTK_WIDGET (graph));
 	gtk_widget_show (GTK_WIDGET (graph));
 }
@@ -165,10 +229,68 @@ gpm_graph_widget_class_init (GpmGraphWidgetClass *class)
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
-	widget_class->expose_event = gpm_graph_widget_expose;
+	widget_class->draw = gpm_graph_widget_draw;
+	object_class->get_property = up_graph_get_property;
+	object_class->set_property = up_graph_set_property;
 	object_class->finalize = gpm_graph_widget_finalize;
 
 	g_type_class_add_private (class, sizeof (GpmGraphWidgetPrivate));
+
+	/* properties */
+	g_object_class_install_property (object_class,
+					 PROP_USE_LEGEND,
+					 g_param_spec_boolean ("use-legend", NULL, NULL,
+							       FALSE,
+							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_USE_GRID,
+					 g_param_spec_boolean ("use-grid", NULL, NULL,
+							       TRUE,
+							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_TYPE_X,
+					 g_param_spec_uint ("type-x", NULL, NULL,
+							    GPM_GRAPH_WIDGET_TYPE_INVALID,
+							    GPM_GRAPH_WIDGET_TYPE_UNKNOWN,
+							    GPM_GRAPH_WIDGET_TYPE_TIME,
+							    G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_TYPE_Y,
+					 g_param_spec_uint ("type-y", NULL, NULL,
+							    GPM_GRAPH_WIDGET_TYPE_INVALID,
+							    GPM_GRAPH_WIDGET_TYPE_UNKNOWN,
+							    GPM_GRAPH_WIDGET_TYPE_PERCENTAGE,
+							    G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_AUTORANGE_X,
+					 g_param_spec_boolean ("autorange-x", NULL, NULL,
+							       TRUE,
+							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_AUTORANGE_Y,
+					 g_param_spec_boolean ("autorange-y", NULL, NULL,
+							       TRUE,
+							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_START_X,
+					 g_param_spec_int ("start-x", NULL, NULL,
+							   G_MININT, G_MAXINT, 0,
+							   G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_START_Y,
+					 g_param_spec_int ("start-y", NULL, NULL,
+							   G_MININT, G_MAXINT, 0,
+							   G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_STOP_X,
+					 g_param_spec_int ("stop-x", NULL, NULL,
+							   G_MININT, G_MAXINT, 60,
+							   G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_STOP_Y,
+					 g_param_spec_int ("stop-y", NULL, NULL,
+							   G_MININT, G_MAXINT, 100,
+							   G_PARAM_READWRITE));
 }
 
 /**
@@ -189,11 +311,11 @@ gpm_graph_widget_init (GpmGraphWidget *graph)
 	graph->priv->stop_y = 100;
 	graph->priv->use_grid = TRUE;
 	graph->priv->use_legend = FALSE;
-	graph->priv->data_list = g_ptr_array_new ();
+	graph->priv->data_list = g_ptr_array_new_with_free_func ((GDestroyNotify) g_ptr_array_unref);
 	graph->priv->plot_list = g_ptr_array_new ();
 	graph->priv->key_data = NULL;
-	graph->priv->axis_type_x = GPM_GRAPH_WIDGET_TYPE_TIME;
-	graph->priv->axis_type_y = GPM_GRAPH_WIDGET_TYPE_PERCENTAGE;
+	graph->priv->type_x = GPM_GRAPH_WIDGET_TYPE_TIME;
+	graph->priv->type_y = GPM_GRAPH_WIDGET_TYPE_PERCENTAGE;
 
 	/* do pango stuff */
 	fontmap = pango_cairo_font_map_get_default ();
@@ -206,6 +328,20 @@ gpm_graph_widget_init (GpmGraphWidget *graph)
 	pango_font_description_free (desc);
 }
 
+/**
+ * gpm_graph_widget_data_clear:
+ **/
+gboolean
+gpm_graph_widget_data_clear (GpmGraphWidget *graph)
+{
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
+
+	g_ptr_array_set_size (graph->priv->data_list, 0);
+	g_ptr_array_set_size (graph->priv->plot_list, 0);
+
+	return TRUE;
+}
+
 /**
  * gpm_graph_widget_finalize:
  * @object: This graph class instance
@@ -216,13 +352,13 @@ gpm_graph_widget_finalize (GObject *object)
 	PangoContext *context;
 	GpmGraphWidget *graph = (GpmGraphWidget*) object;
 
-	/* clear key */
+	/* clear key and data */
 	gpm_graph_widget_key_data_clear (graph);
+	gpm_graph_widget_data_clear (graph);
 
 	/* free data */
-	g_ptr_array_foreach (graph->priv->data_list, (GFunc) g_object_unref, NULL);
-	g_ptr_array_free (graph->priv->data_list, TRUE);
-	g_ptr_array_free (graph->priv->plot_list, TRUE);
+	g_ptr_array_unref (graph->priv->data_list);
+	g_ptr_array_unref (graph->priv->plot_list);
 
 	context = pango_layout_get_context (graph->priv->layout);
 	g_object_unref (graph->priv->layout);
@@ -230,35 +366,32 @@ gpm_graph_widget_finalize (GObject *object)
 	G_OBJECT_CLASS (gpm_graph_widget_parent_class)->finalize (object);
 }
 
-/**
- * gpm_graph_widget_data_clear:
- **/
-gboolean
-gpm_graph_widget_data_clear (GpmGraphWidget *graph)
-{
-	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
-
-	g_ptr_array_foreach (graph->priv->data_list, (GFunc) g_object_unref, NULL);
-	g_ptr_array_set_size (graph->priv->data_list, 0);
-	g_ptr_array_set_size (graph->priv->plot_list, 0);
-
-	return TRUE;
-}
-
 /**
  * gpm_graph_widget_data_assign:
  * @graph: This class instance
+ * @data: an array of GpmPointObj's
  *
  * Sets the data for the graph
  **/
 gboolean
-gpm_graph_widget_data_assign (GpmGraphWidget *graph, GpmGraphWidgetPlot plot, EggObjList *data)
+gpm_graph_widget_data_assign (GpmGraphWidget *graph, GpmGraphWidgetPlot plot, GPtrArray *data)
 {
+	GPtrArray *copy;
+	GpmPointObj *obj;
+	guint i;
+
 	g_return_val_if_fail (data != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
 
+	/* make a deep copy */
+	copy = g_ptr_array_new_with_free_func ((GDestroyNotify) gpm_point_obj_free);
+	for (i=0; i<data->len; i++) {
+		obj = gpm_point_obj_copy (g_ptr_array_index (data, i));
+		g_ptr_array_add (copy, obj);
+	}
+
 	/* get the new data */
-	g_ptr_array_add (graph->priv->data_list, g_object_ref (data));
+	g_ptr_array_add (graph->priv->data_list, copy);
 	g_ptr_array_add (graph->priv->plot_list, GUINT_TO_POINTER(plot));
 
 	/* refresh */
@@ -346,7 +479,8 @@ gpm_get_axis_label (GpmGraphWidgetType axis, gfloat value)
 static void
 gpm_graph_widget_draw_grid (GpmGraphWidget *graph, cairo_t *cr)
 {
-	gfloat a, b;
+	guint i;
+	gfloat b;
 	gdouble dotted[] = {1., 2.};
 	gfloat divwidth  = (gfloat)graph->priv->box_width / 10.0f;
 	gfloat divheight = (gfloat)graph->priv->box_height / 10.0f;
@@ -358,16 +492,16 @@ gpm_graph_widget_draw_grid (GpmGraphWidget *graph, cairo_t *cr)
 
 	/* do vertical lines */
 	cairo_set_source_rgb (cr, 0.1, 0.1, 0.1);
-	for (a=1; a<10; a++) {
-		b = graph->priv->box_x + (a * divwidth);
+	for (i=1; i<10; i++) {
+		b = graph->priv->box_x + ((gfloat) i * divwidth);
 		cairo_move_to (cr, (gint)b + 0.5f, graph->priv->box_y);
 		cairo_line_to (cr, (gint)b + 0.5f, graph->priv->box_y + graph->priv->box_height);
 		cairo_stroke (cr);
 	}
 
 	/* do horizontal lines */
-	for (a=1; a<10; a++) {
-		b = graph->priv->box_y + (a * divheight);
+	for (i=1; i<10; i++) {
+		b = graph->priv->box_y + ((gfloat) i * divheight);
 		cairo_move_to (cr, graph->priv->box_x, (gint)b + 0.5f);
 		cairo_line_to (cr, graph->priv->box_x + graph->priv->box_width, (int)b + 0.5f);
 		cairo_stroke (cr);
@@ -386,7 +520,8 @@ gpm_graph_widget_draw_grid (GpmGraphWidget *graph, cairo_t *cr)
 static void
 gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 {
-	gfloat a, b;
+	guint i;
+	gfloat b;
 	gchar *text;
 	gfloat value;
 	gfloat divwidth  = (gfloat)graph->priv->box_width / 10.0f;
@@ -401,17 +536,17 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 
 	/* do x text */
 	cairo_set_source_rgb (cr, 0, 0, 0);
-	for (a=0; a<11; a++) {
-		b = graph->priv->box_x + (a * divwidth);
-		value = ((length_x / 10.0f) * (gfloat) a) + (gfloat) graph->priv->start_x;
-		text = gpm_get_axis_label (graph->priv->axis_type_x, value);
+	for (i=0; i<11; i++) {
+		b = graph->priv->box_x + ((gfloat) i * divwidth);
+		value = ((length_x / 10.0f) * (gfloat) i) + (gfloat) graph->priv->start_x;
+		text = gpm_get_axis_label (graph->priv->type_x, value);
 
 		pango_layout_set_text (graph->priv->layout, text, -1);
 		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
 		/* have data points 0 and 10 bounded, but 1..9 centered */
-		if (a == 0)
+		if (i == 0)
 			offsetx = 2.0;
-		else if (a == 10)
+		else if (i == 10)
 			offsetx = ink_rect.width;
 		else
 			offsetx = (ink_rect.width / 2.0f);
@@ -424,18 +559,18 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 	}
 
 	/* do y text */
-	for (a=0; a<11; a++) {
-		b = graph->priv->box_y + (a * divheight);
-		value = ((gfloat) length_y / 10.0f) * (10 - a) + graph->priv->start_y;
-		text = gpm_get_axis_label (graph->priv->axis_type_y, value);
+	for (i=0; i<11; i++) {
+		b = graph->priv->box_y + ((gfloat) i * divheight);
+		value = ((gfloat) length_y / 10.0f) * (10 - (gfloat) i) + graph->priv->start_y;
+		text = gpm_get_axis_label (graph->priv->type_y, value);
 
 		pango_layout_set_text (graph->priv->layout, text, -1);
 		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
 
 		/* have data points 0 and 10 bounded, but 1..9 centered */
-		if (a == 10)
+		if (i == 10)
 			offsety = 0;
-		else if (a == 0)
+		else if (i == 0)
 			offsety = ink_rect.height;
 		else
 			offsety = (ink_rect.height / 2.0f);
@@ -459,19 +594,17 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 static guint
 gpm_graph_widget_get_y_label_max_width (GpmGraphWidget *graph, cairo_t *cr)
 {
-	gfloat a, b;
+	guint i;
 	gchar *text;
 	gint value;
-	gfloat divheight = (gfloat)graph->priv->box_height / 10.0f;
 	gint length_y = graph->priv->stop_y - graph->priv->start_y;
 	PangoRectangle ink_rect, logical_rect;
 	guint biggest = 0;
 
 	/* do y text */
-	for (a=0; a<11; a++) {
-		b = graph->priv->box_y + (a * divheight);
-		value = (length_y / 10) * (10 - a) + graph->priv->start_y;
-		text = gpm_get_axis_label (graph->priv->axis_type_y, value);
+	for (i=0; i<11; i++) {
+		value = (length_y / 10) * (10 - (gfloat) i) + graph->priv->start_y;
+		text = gpm_get_axis_label (graph->priv->type_y, value);
 		pango_layout_set_text (graph->priv->layout, text, -1);
 		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
 		if (ink_rect.width > (gint) biggest)
@@ -482,7 +615,7 @@ gpm_graph_widget_get_y_label_max_width (GpmGraphWidget *graph, cairo_t *cr)
 }
 
 /**
- * gpm_graph_widget_auto_range:
+ * gpm_graph_widget_autorange_x:
  * @graph: This class instance
  *
  * Autoranges the graph axis depending on the axis type, and the maximum
@@ -490,15 +623,12 @@ gpm_graph_widget_get_y_label_max_width (GpmGraphWidget *graph, cairo_t *cr)
  * resolution but also a number that scales "well" to a 10x10 grid.
  **/
 static void
-gpm_graph_widget_auto_range (GpmGraphWidget *graph)
+gpm_graph_widget_autorange_x (GpmGraphWidget *graph)
 {
 	gfloat biggest_x = G_MINFLOAT;
-	gfloat biggest_y = G_MINFLOAT;
 	gfloat smallest_x = G_MAXFLOAT;
-	gfloat smallest_y = G_MAXFLOAT;
 	guint rounding_x = 1;
-	guint rounding_y = 1;
-	EggObjList *data;
+	GPtrArray *data;
 	GpmPointObj *point;
 	guint i, j;
 	guint len = 0;
@@ -516,10 +646,102 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 
 	/* no data in any array */
 	if (len == 0) {
-		egg_debug ("no data");
+		g_debug ("no data");
 		graph->priv->start_x = 0;
-		graph->priv->start_y = 0;
 		graph->priv->stop_x = 10;
+		return;
+	}
+
+	/* get the range for the graph */
+	for (j=0; j<array->len; j++) {
+		data = g_ptr_array_index (array, j);
+		for (i=0; i < data->len; i++) {
+			point = (GpmPointObj *) g_ptr_array_index (data, i);
+			if (point->x > biggest_x)
+				biggest_x = point->x;
+			if (point->x < smallest_x)
+				smallest_x = point->x;
+		}
+	}
+	g_debug ("Data range is %f<x<%f", smallest_x, biggest_x);
+	/* don't allow no difference */
+	if (biggest_x - smallest_x < 0.0001) {
+		biggest_x++;
+		smallest_x--;
+	}
+
+	if (graph->priv->type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+		rounding_x = 10;
+	} else if (graph->priv->type_x == GPM_GRAPH_WIDGET_TYPE_FACTOR) {
+		rounding_x = 1;
+	} else if (graph->priv->type_x == GPM_GRAPH_WIDGET_TYPE_POWER) {
+		rounding_x = 10;
+	} else if (graph->priv->type_x == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+		rounding_x = 1000;
+	} else if (graph->priv->type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		if (biggest_x-smallest_x < 150)
+			rounding_x = 150;
+		else if (biggest_x-smallest_x < 5*60)
+			rounding_x = 5 * 60;
+		else
+			rounding_x = 10 * 60;
+	}
+
+	graph->priv->start_x = gpm_precision_round_down (smallest_x, rounding_x);
+	graph->priv->stop_x = gpm_precision_round_up (biggest_x, rounding_x);
+
+	g_debug ("Processed(1) range is %i<x<%i",
+		   graph->priv->start_x, graph->priv->stop_x);
+
+	/* if percentage, and close to the end points, then extend */
+	if (graph->priv->type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+		if (graph->priv->stop_x >= 90)
+			graph->priv->stop_x = 100;
+		if (graph->priv->start_x > 0 && graph->priv->start_x <= 10)
+			graph->priv->start_x = 0;
+	} else if (graph->priv->type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		if (graph->priv->start_x > 0 && graph->priv->start_x <= 60*10)
+			graph->priv->start_x = 0;
+	}
+
+	g_debug ("Processed range is %i<x<%i",
+		   graph->priv->start_x, graph->priv->stop_x);
+}
+
+/**
+ * gpm_graph_widget_autorange_y:
+ * @graph: This class instance
+ *
+ * Autoranges the graph axis depending on the axis type, and the maximum
+ * value of the data. We have to be careful to choose a number that gives good
+ * resolution but also a number that scales "well" to a 10x10 grid.
+ **/
+static void
+gpm_graph_widget_autorange_y (GpmGraphWidget *graph)
+{
+	gfloat biggest_y = G_MINFLOAT;
+	gfloat smallest_y = G_MAXFLOAT;
+	guint rounding_y = 1;
+	GPtrArray *data;
+	GpmPointObj *point;
+	guint i, j;
+	guint len = 0;
+	GPtrArray *array;
+
+	array = graph->priv->data_list;
+
+	/* find out if we have no data */
+	for (j=0; j<array->len; j++) {
+		data = g_ptr_array_index (array, j);
+		len = data->len;
+		if (len > 0)
+			break;
+	}
+
+	/* no data in any array */
+	if (len == 0) {
+		g_debug ("no data");
+		graph->priv->start_y = 0;
 		graph->priv->stop_y = 10;
 		return;
 	}
@@ -528,53 +750,29 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 	for (j=0; j<array->len; j++) {
 		data = g_ptr_array_index (array, j);
 		for (i=0; i < data->len; i++) {
-			point = (GpmPointObj *) egg_obj_list_index (data, i);
-			if (point->x > biggest_x)
-				biggest_x = point->x;
+			point = (GpmPointObj *) g_ptr_array_index (data, i);
 			if (point->y > biggest_y)
 				biggest_y = point->y;
-			if (point->x < smallest_x)
-				smallest_x = point->x;
 			if (point->y < smallest_y)
 				smallest_y = point->y;
 		}
 	}
-	egg_debug ("Data range is %f<x<%f, %f<y<%f", smallest_x, biggest_x, smallest_y, biggest_y);
+	g_debug ("Data range is %f<y<%f", smallest_y, biggest_y);
 	/* don't allow no difference */
-	if (biggest_x - smallest_x < 0.0001) {
-		biggest_x++;
-		smallest_x--;
-	}
 	if (biggest_y - smallest_y < 0.0001) {
 		biggest_y++;
 		smallest_y--;
 	}
 
-	if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
-		rounding_x = 10;
-	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_FACTOR) {
-		rounding_x = 1;
-	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_POWER) {
-		rounding_x = 10;
-	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
-		rounding_x = 1000;
-	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
-		if (biggest_x-smallest_x < 150)
-			rounding_x = 150;
-		else if (biggest_x-smallest_x < 5*60)
-			rounding_x = 5 * 60;
-		else
-			rounding_x = 10 * 60;
-	}
-	if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+	if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
 		rounding_y = 10;
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_FACTOR) {
+	} else if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_FACTOR) {
 		rounding_y = 1;
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
+	} else if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
 		rounding_y = 10;
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+	} else if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
 		rounding_y = 1000;
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
+	} else if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
 		if (biggest_y-smallest_y < 150)
 			rounding_y = 150;
 		else if (biggest_y < 5*60)
@@ -583,45 +781,31 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 			rounding_y = 10 * 60;
 	}
 
-	graph->priv->start_x = egg_precision_round_down (smallest_x, rounding_x);
-	graph->priv->start_y = egg_precision_round_down (smallest_y, rounding_y);
-	graph->priv->stop_x = egg_precision_round_up (biggest_x, rounding_x);
-	graph->priv->stop_y = egg_precision_round_up (biggest_y, rounding_y);
+	graph->priv->start_y = gpm_precision_round_down (smallest_y, rounding_y);
+	graph->priv->stop_y = gpm_precision_round_up (biggest_y, rounding_y);
 
 	/* a factor graph always is centered around zero */
-	if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_FACTOR) {
+	if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_FACTOR) {
 		if (abs (graph->priv->stop_y) > abs (graph->priv->start_y))
 			graph->priv->start_y = -graph->priv->stop_y;
 		else
 			graph->priv->stop_y = -graph->priv->start_y;
 	}
 
-	egg_debug ("Processed(1) range is %i<x<%i, %i<y<%i",
-		   graph->priv->start_x, graph->priv->stop_x,
+	g_debug ("Processed(1) range is %i<y<%i",
 		   graph->priv->start_y, graph->priv->stop_y);
 
-	/* if percentage, and close to the end points, then extend */
-	if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
-		if (graph->priv->stop_x >= 90)
-			graph->priv->stop_x = 100;
-		if (graph->priv->start_x > 0 && graph->priv->start_x <= 10)
-			graph->priv->start_x = 0;
-	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
-		if (graph->priv->start_x > 0 && graph->priv->start_x <= 60*10)
-			graph->priv->start_x = 0;
-	}
-	if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+	if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
 		if (graph->priv->stop_y >= 90)
 			graph->priv->stop_y = 100;
 		if (graph->priv->start_y > 0 && graph->priv->start_y <= 10)
 			graph->priv->start_y = 0;
-	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
+	} else if (graph->priv->type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
 		if (graph->priv->start_y <= 60*10)
 			graph->priv->start_y = 0;
 	}
 
-	egg_debug ("Processed range is %i<x<%i, %i<y<%i",
-		   graph->priv->start_x, graph->priv->stop_x,
+	g_debug ("Processed range is %i<y<%i",
 		   graph->priv->start_y, graph->priv->stop_y);
 }
 
@@ -634,7 +818,7 @@ static void
 gpm_graph_widget_set_color (cairo_t *cr, guint32 color)
 {
 	guint8 r, g, b;
-	egg_color_to_rgb (color, &r, &g, &b);
+	gpm_color_to_rgb (color, &r, &g, &b);
 	cairo_set_source_rgb (cr, ((gdouble) r)/256.0f, ((gdouble) g)/256.0f, ((gdouble) b)/256.0f);
 }
 
@@ -709,14 +893,14 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 {
 	gfloat oldx, oldy;
 	gfloat newx, newy;
-	EggObjList *data;
+	GPtrArray *data;
 	GPtrArray *array;
 	GpmGraphWidgetPlot plot;
 	GpmPointObj *point;
 	guint i, j;
 
 	if (graph->priv->data_list->len == 0) {
-		egg_debug ("no data");
+		g_debug ("no data");
 		return;
 	}
 	cairo_save (cr);
@@ -731,14 +915,15 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 		plot = GPOINTER_TO_UINT (g_ptr_array_index (graph->priv->plot_list, j));
 
 		/* get the very first point so we can work out the old */
-		point = (GpmPointObj *) egg_obj_list_index (data, 0);
+		point = (GpmPointObj *) g_ptr_array_index (data, 0);
 		oldx = 0;
 		oldy = 0;
 		gpm_graph_widget_get_pos_on_graph (graph, point->x, point->y, &oldx, &oldy);
-		gpm_graph_widget_draw_dot (cr, oldx, oldy, point->color);
+		if (plot == GPM_GRAPH_WIDGET_PLOT_POINTS || plot == GPM_GRAPH_WIDGET_PLOT_BOTH)
+			gpm_graph_widget_draw_dot (cr, oldx, oldy, point->color);
 
 		for (i=1; i < data->len; i++) {
-			point = (GpmPointObj *) egg_obj_list_index (data, i);
+			point = (GpmPointObj *) g_ptr_array_index (data, i);
 
 			gpm_graph_widget_get_pos_on_graph (graph, point->x, point->y, &newx, &newy);
 
@@ -806,18 +991,18 @@ gpm_graph_widget_draw_legend (GpmGraphWidget *graph, gint x, gint y, gint width,
 {
 	cairo_t *cr = graph->priv->cr;
 	gint y_count;
-	guint a;
+	guint i;
 	GpmGraphWidgetKeyData *keydataitem;
 
 	gpm_graph_widget_draw_bounding_box (cr, x, y, width, height);
 	y_count = y + 10;
 
 	/* add the line colors to the legend */
-	for (a=0; a<g_slist_length (graph->priv->key_data); a++) {
-		keydataitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, a);
+	for (i=0; i<g_slist_length (graph->priv->key_data); i++) {
+		keydataitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, i);
 		if (keydataitem == NULL) {
 			/* this shouldn't ever happen */
-			egg_warning ("keydataitem NULL!");
+			g_warning ("keydataitem NULL!");
 			break;
 		}
 		gpm_graph_widget_draw_legend_line (cr, x + 8, y_count, keydataitem->color);
@@ -843,7 +1028,7 @@ static gboolean
 gpm_graph_widget_legend_calculate_size (GpmGraphWidget *graph, cairo_t *cr,
 					guint *width, guint *height)
 {
-	guint a;
+	guint i;
 	PangoRectangle ink_rect, logical_rect;
 	GpmGraphWidgetKeyData *keydataitem;
 
@@ -854,8 +1039,8 @@ gpm_graph_widget_legend_calculate_size (GpmGraphWidget *graph, cairo_t *cr,
 	*height = 0;
 
 	/* add the line colors to the legend */
-	for (a=0; a<g_slist_length (graph->priv->key_data); a++) {
-		keydataitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, a);
+	for (i=0; i<g_slist_length (graph->priv->key_data); i++) {
+		keydataitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, i);
 		*height = *height + GPM_GRAPH_WIDGET_LEGEND_SPACING;
 		pango_layout_set_text (graph->priv->layout, keydataitem->desc, -1);
 		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
@@ -875,15 +1060,16 @@ gpm_graph_widget_legend_calculate_size (GpmGraphWidget *graph, cairo_t *cr,
 }
 
 /**
- * gpm_graph_widget_draw_graph:
+ * gpm_graph_widget_draw:
  * @graph: This class instance
- * @cr: Cairo drawing context
+ * @event: The expose event
  *
- * Draw the complete graph, with the box, the grid, the labels and the line.
+ * Just repaint the entire graph widget on expose.
  **/
-static void
-gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
+static gboolean
+gpm_graph_widget_draw (GtkWidget *widget, cairo_t *cr)
 {
+	GtkAllocation allocation;
 	gint legend_x = 0;
 	gint legend_y = 0;
 	guint legend_height = 0;
@@ -891,29 +1077,33 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	gfloat data_x;
 	gfloat data_y;
 
-	GpmGraphWidget *graph = (GpmGraphWidget*) graph_widget;
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
+	GpmGraphWidget *graph = (GpmGraphWidget*) widget;
+	g_return_val_if_fail (graph != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
 
 	gpm_graph_widget_legend_calculate_size (graph, cr, &legend_width, &legend_height);
 	cairo_save (cr);
 
 	/* we need this so we know the y text */
-	gpm_graph_widget_auto_range (graph);
+	if (graph->priv->autorange_x)
+		gpm_graph_widget_autorange_x (graph);
+	if (graph->priv->autorange_y)
+		gpm_graph_widget_autorange_y (graph);
 
 	graph->priv->box_x = gpm_graph_widget_get_y_label_max_width (graph, cr) + 10;
 	graph->priv->box_y = 5;
 
-	graph->priv->box_height = graph_widget->allocation.height - (20 + graph->priv->box_y);
+	gtk_widget_get_allocation (widget, &allocation);
+	graph->priv->box_height = allocation.height - (20 + graph->priv->box_y);
 
 	/* make size adjustment for legend */
 	if (graph->priv->use_legend && legend_height > 0) {
-		graph->priv->box_width = graph_widget->allocation.width -
+		graph->priv->box_width = allocation.width -
 					 (3 + legend_width + 5 + graph->priv->box_x);
 		legend_x = graph->priv->box_x + graph->priv->box_width + 6;
 		legend_y = graph->priv->box_y;
 	} else {
-		graph->priv->box_width = graph_widget->allocation.width -
+		graph->priv->box_width = allocation.width -
 					 (3 + graph->priv->box_x);
 	}
 
@@ -936,31 +1126,6 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 		gpm_graph_widget_draw_legend (graph, legend_x, legend_y, legend_width, legend_height);
 
 	cairo_restore (cr);
-}
-
-/**
- * gpm_graph_widget_expose:
- * @graph: This class instance
- * @event: The expose event
- *
- * Just repaint the entire graph widget on expose.
- **/
-static gboolean
-gpm_graph_widget_expose (GtkWidget *graph, GdkEventExpose *event)
-{
-	cairo_t *cr;
-
-	/* get a cairo_t */
-	cr = gdk_cairo_create (graph->window);
-	cairo_rectangle (cr,
-			 event->area.x, event->area.y,
-			 event->area.width, event->area.height);
-	cairo_clip (cr);
-	((GpmGraphWidget *)graph)->priv->cr = cr;
-
-	gpm_graph_widget_draw_graph (graph, cr);
-
-	cairo_destroy (cr);
 	return FALSE;
 }
 
